/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

/* This file has been modified by Markus Lorch (markus@lorch.net) to only
   display the top ten domains (based on transfer) in the domain pie chart
   - this solves an issue with too many names in the legend if one has more
   than 14 domains or so */



#ifndef GRAPHS_H
#include "graphs.h"
#endif
#include <sys/types.h>
#ifndef STUFF_H
#include "stuff.h"
#endif
#include <math.h>
#include <time.h>
#include <string.h>
#define PI 3.14159265


Images::Images (Config *config_p)
{
  imgptr = NULL;
  config = config_p;
}

Images::~Images ()
{
  imgptr = NULL;
}

gdImagePtr Images::init_graph (char *title, int xsize, int ysize)
{ gdImagePtr color;
  FILE *f;
  int back;
  
  imgptr = gdImageCreate(xsize,ysize);
  // Hier nun 255 Farben laden
  // wird aus base.png geladen
  f = fopen (config->pfile,"r");
  color = gdImageCreateFromPng (f);
  gdImagePaletteCopy (imgptr,color);
  gdImageDestroy (color);
  fclose (f);
  white = gdImageColorClosest(imgptr, 255,255,255);
  black = gdImageColorClosest(imgptr, 0,0,0);
  green = gdImageColorClosest(imgptr, 51, 102, 153);
  back = gdImageColorClosest(imgptr,255,250,205);
  orange = gdImageColorClosest(imgptr, 150, 50, 50);
  // Den Hintergrund anfnglich wei zeichnen
  gdImageColorTransparent(imgptr,white);
  gdImageFilledRectangle(imgptr,0,0,xsize,ysize, white);
  return imgptr;
}

void Images::save_image (char *save_file)
{ FILE *imgfile;
  char *fullpath;
  
  fullpath = (char *)malloc(200);
  strcpy (fullpath,config->imgdir);
  strcat (fullpath,save_file);
  if ((imgfile = fopen(fullpath, "wb")) != NULL)
  {
    gdImagePng(imgptr,imgfile);
    fclose (imgfile);
  }
  free (fullpath);
}

char *Images::month_usage (unsigned long usage[12],char *domainname)
{ double maximum;
  double maxhigh=269;
  double pix;
  int i,high;
  int offset=40;
  unsigned char *buff;
  char s[100]="Overview for year ";
  struct tm *zeit;
  char  *save_file;
  time_t *tempzeit=new(time_t);
  
  buff = (unsigned char *)malloc(32);
  // Initialisieren
  init_graph("Monthly Usage",560,300);
  // Die Achsen zeichnen
  gdImageLine(imgptr, 30, 0, 30, 300, black);	
  gdImageLine(imgptr, 0,270,560,270, black);
  // Die Beschriftung erstellen
  time(tempzeit);
  zeit = localtime (tempzeit);
  strcat (s,(char *)itoa(zeit->tm_year+1900,buff,10));
  gdImageString(imgptr, gdFontLarge,imgptr->sx / 2 - (strlen(s) * gdFontLarge->w / 2),5,(unsigned char *)s,black);
  // Das Maximum bestimmen
  maximum = usage[0];
  for (i=1;i<12;i++)
    if (usage[i] > maximum) maximum = usage[i];
  if (maximum > 0)
    pix = (maxhigh-30) / maximum;
  else
    pix = 0;
  // Die Balken zeichnen
  for (i=0;i<12;i++) {
    high = (int) rint(pix * usage[i]);
    // Und zeichnen
    gdImageFilledRectangle (imgptr,offset+i*42,(int)maxhigh-high,offset+i*42+30,269,green);
    buff = itoa(i+1,buff,10);
    gdImageString(imgptr, gdFontSmall,offset+i*42+15-((gdFontSmall->w / 2)*strlen((char *)buff)),275,buff,black);
    buff = itoa(usage[i],buff,10);
    gdImageString(imgptr, gdFontSmall,offset+i*42+15-((gdFontSmall->w / 2)*strlen((char *)buff)),(int)maxhigh-high-12,buff,black);
  }
  // Speichern
  save_file = (char *)malloc(strlen(domainname)+4+4+1);
  strcpy (save_file,domainname);
  strcat (save_file,(char *)itoa(zeit->tm_year+1900,buff,10));
 
  strcat (save_file,".png");
  
  printf ("%s\n",save_file);
  save_image (save_file);
  free (buff);
  free (tempzeit);
  return save_file;
}

void Images::hourly_usage (int usage[2][24],struct tm *start)
{ double maximum;
  double d_maximum;
  int i;
  int high;
  int d_high;
  double maxhigh=269;	// -1 fr untere Linie
  int offset=40;
  unsigned char *buff;
  char buf[20];
  char s[100]="bersicht fr ";
  char *zeit;
  char save_file[100];
  double pix;
  double d_pix;
  
  buff = (unsigned char *)malloc(32);
  // Initialisieren
  init_graph("Hourly Usage",560,300);
  // Die Achsen zeichnen
  gdImageLine(imgptr, 30, 0, 30, 300, black);	
  gdImageLine(imgptr, 0,270,560,270, black);
  // Die Beschriftung erstellen
  zeit = (char *) malloc(40);
  zeit = asctime(start);
  zeit[strlen(zeit)-1] = 0;
  strcat (s,zeit);
  gdImageString(imgptr, gdFontLarge,imgptr->sx / 2 - (strlen(s) * gdFontLarge->w / 2),5,(unsigned char *)s,black);
  // Das Maximum bestimmen
  maximum = usage[0][0];
  for (i=1;i<24;i++)
    if (usage[0][i] > maximum) maximum = usage[0][i];
  pix = (maxhigh-30) / maximum;
  d_maximum = usage[1][0];
  for (i=1;i<24;i++)
    if (usage[1][i] > d_maximum) d_maximum = usage[1][i];
  d_pix = (maxhigh-30) / d_maximum;
  // Die Balken zeichnen
  for (i=0;i<24;i++) {
    high = (int) rint(pix * usage[0][i]);
    d_high = (int) rint(d_pix * usage[1][i]);
    // Und zeichnen
    gdImageFilledRectangle (imgptr,offset+i*21,(int)maxhigh-high,offset+i*21+10,269,green);
//    gdImageFilledRectangle (imgptr,offset+i*21+6,(int)maxhigh-high,offset+i*21+10+6,269,orange);
    buff = itoa(i,buff,10);
    gdImageString(imgptr, gdFontSmall,offset+i*21+7-((gdFontSmall->w / 2)*strlen((char *)buff)),275,buff,black);
    buff = itoa(usage[0][i],buff,10);
    gdImageString(imgptr, gdFontSmall,offset+i*21+7-((gdFontSmall->w / 2)*strlen((char *)buff)),(int)maxhigh-d_high-12,buff,black);
  }
  // Speichern
  save_file[0] = 0;
  strcpy (save_file,"Daily-");
  itoa(start->tm_mday,buf,10);
  strcat (save_file,buf);
  itoa(start->tm_mon,buf,10);
  strcat (save_file,buf);
  itoa(start->tm_year,buf,10);
  strcat (save_file,buf);
  strcat (save_file,".png");
  printf ("%s\n",save_file);
  save_image (save_file);
  free (buff);
  free (zeit);
}

// Diese Funktion erzeugt eine Graphik die eine bersicht ber die Domains gibt
// Speichert die Graphik und gibt den Dateinamen zurck
// Aufgetielt wird auf die sumbytes
char *Images::domain_view (Domains *domain)
{ char *filename;
  double gesbytes;
  double verbytes;
  Domains *aktdomain;
  int aangle,eangle;
  int i,anzahl,y,x;
  double percent;
  double t[10]; // top ten byte usages (top ten domains) (ML)
  int m,n; // counters for top ten sort loop  (ML)
   
  // Am Anfang mal das leere Bild erstellen
  init_graph ("Domains",600,300);
  // Die gesbytes berechnen
  aktdomain = domain;
  gesbytes = 0;
  anzahl = 0;
  t[0]=t[1]=t[2]=t[3]=t[4]=t[5]=t[6]=t[7]=t[8]=t[9]=0;
  while (aktdomain != NULL) {
    gesbytes += aktdomain->bytes;
    /* the following was added by Markus Lorch (ML) <markus@lorch.net> */
    //printf("domain usage for %s is: %f \n", (unsigned char *)aktdomain->domainname, aktdomain->bytes); 
    for(m=0;m<10;m++)
       if (aktdomain->bytes > t[m]) {           // did we find a new top ten position ?
           for (n=9; n>m; n--) t[n] = t[n-1];   // if so, shift remainder back
           t[m] = aktdomain->bytes;		// an enter the new top position
           //printf("domain entered at position %i\n",m);
           break;
         }
    //printf("Domain Top Ten : "); for(m=0;m<10;m++) printf("%i: %f ", m, t[m]); printf("\n");
    /* end markus@lorch.net */
    if (aktdomain->islocal)
      anzahl++;
    aktdomain = aktdomain->next;
  }
  anzahl++;
  // Dann zu zeichnen beginnen
  aktdomain = domain;
  aangle = 0;
  verbytes = 0;
  i = 10;
  black = gdImageColorClosest (imgptr,0,0,0);
  while (aktdomain != NULL) {
    if ((aktdomain->islocal) && (aktdomain->bytes >= t[9])) {  // is local and is in top ten (ML)
      percent = aktdomain->bytes / gesbytes;
      verbytes += aktdomain->bytes;
      eangle = (int) (aangle + 360 * percent);
      // zeichnen
      i++;
      DrawPie(imgptr,200,150,130,aangle,eangle,i);
      // Den verweis auch noch machen
      if(anzahl<10)
        y = (300-20*anzahl) / 2 + (i-11)*20;
      else
        y = (300-20*10) / 2 + (i-11)*20; // we will only show the top ten if we have more then 10 domains (ML)
      x = 360;
      gdImageFilledRectangle(imgptr, x, y, x+15, y+15, i);
      gdImageString(imgptr, gdFontSmall,x+20,y+10- (gdFontSmall->h / 2),(unsigned char *)aktdomain->domainname, black);      
      aangle = eangle;      
    }
    aktdomain = aktdomain->next;
  }
  // Und das was berbleibt ist von relaying
  percent = (gesbytes-verbytes) / gesbytes;
  eangle = 360;
  // zeichnen
  i++;
  y = (300-20*anzahl) / 2 + (i-11)*20;
  i = gdImageColorClosest (imgptr,255,0,0);
  DrawPie(imgptr,200,150,130,aangle,eangle,i);
  x = 360;
  gdImageFilledRectangle(imgptr, x, y, x+15, y+15, i);
  gdImageString(imgptr, gdFontSmall,x+20,y+10- (gdFontSmall->h / 2),(unsigned char *)"relay", black);      

  filename = (char *)malloc(strlen("domains.png")+1);
  strcpy (filename,"domains.png");
  save_image (filename);
  return filename; 
}

void Images::DrawPie (gdImagePtr image,int mx,int my,int r,int aangle,int eangle,int color)
{ int p1x,p2x,p1y,p2y,p3x,p3y;
  int black;
  double a,e,m;
    
  black = gdImageColorClosest (image,0,0,0);
  // Den Kreis zeichnen
  a = aangle / 180.0 * PI;
  e = eangle / 180.0 * PI;
  m = (((eangle-aangle) / 2.0) + aangle) / 180.0 * PI;
  gdImageArc (image,mx,my,r*2,(int)(r*2.0/1.1),0,360,black);
  gdImageArc (image,mx,my+10,r*2-2,(int)((r*2.0-2)/1.1),2,178,black);
  gdImageFillToBorder(image,mx,my+(int)(((r)+1)/1.1),black,black);
  p1x = (int) (sin(a) * r);
  p2x = (int) (sin(e) * r);
  p1y = (int) (cos(a) * (r/1.1));
  p2y = (int) (cos(e) * (r/1.1));
  p3x = (int) (sin(m) * (r-2));
  p3y = (int) (cos(m) * ((r-2)/1.1));
  // die zwei linien zeichnen
  // mit schwarz
  gdImageLine (image,mx,my,mx+p1x,my+p1y,black);
  gdImageLine (image,mx,my,mx+p2x,my+p2y,black);
  // Jetzt das innere Fllen
  gdImageFillToBorder (image,mx + p3x,my+p3y,black,color);
}
